"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatResult = void 0;
const logger_1 = require("../../logger");
const format_1 = require("../lint/format");
const myLogger = logger_1.logger.sublogger('GuardFormat');
function formatResult({ success, ruleSuccess, manifestPackage, k8sSchemaInfo, rulesRuntime, lintCommandData }) {
    var _a;
    const lintResult = (0, format_1.formatResult)(lintCommandData);
    myLogger.info("Success: %s", success);
    myLogger.info("RuleSuccess: %s", ruleSuccess);
    myLogger.info("LintResult.success: %s", lintResult.success);
    const result = {
        success: success,
        ruleSuccess: ruleSuccess,
        lintResult: lintResult,
        rules: [],
        counters: {
            rules: {
                total: rulesRuntime.rules.length,
                passed: 0,
                failed: 0,
                withErrors: 0,
                withWarnings: 0
            },
            manifests: {
                total: manifestPackage.manifests.length,
                processed: 0,
                passed: 0,
                withErrors: 0,
                withWarnings: 0
            }
        }
    };
    for (const rule of rulesRuntime.rules) {
        const ruleResult = {
            source: rule.rule.source,
            kind: rule.rule.kind,
            namespace: rule.rule.namespace,
            rule: rule.rule.name,
            compiled: rule.isCompiled && !rule.hasRuntimeErrors,
            pass: true,
            hasViolationErrors: false,
            hasViolationWarnings: false,
            passed: rule.passed.map(x => ({
                manifest: x.id,
                source: x.source.source
            }))
        };
        ruleResult.errors = (_a = rule.ruleErrors.map(x => x.msg)) !== null && _a !== void 0 ? _a : [];
        if (!ruleResult.compiled) {
            result.counters.rules.failed++;
        }
        if (rule.violations.length > 0) {
            ruleResult.violations = [];
            for (const violation of rule.violations) {
                if (violation.hasErrors) {
                    ruleResult.pass = false;
                    ruleResult.hasViolationErrors = true;
                }
                if (violation.hasWarnings) {
                    ruleResult.hasViolationWarnings = true;
                }
                ruleResult.violations.push({
                    manifest: violation.manifest.id,
                    source: violation.manifest.source.source,
                    errors: violation.errors,
                    warnings: violation.warnings,
                });
            }
        }
        if (!ruleResult.compiled) {
            result.counters.rules.failed++;
        }
        if (ruleResult.passed) {
            result.counters.rules.passed++;
        }
        if (ruleResult.hasViolationErrors) {
            result.counters.rules.withErrors++;
        }
        if (ruleResult.hasViolationWarnings) {
            result.counters.rules.withWarnings++;
        }
        result.rules.push(ruleResult);
    }
    for (const manifest of manifestPackage.manifests) {
        if (manifest.rules.processed) {
            result.counters.manifests.processed++;
            if (manifest.rules.errors) {
                result.counters.manifests.withErrors++;
            }
            else {
                result.counters.manifests.passed++;
            }
            if (manifest.rules.warnings) {
                result.counters.manifests.withWarnings++;
            }
        }
    }
    return result;
}
exports.formatResult = formatResult;
//# sourceMappingURL=format.js.map