"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.outputGuardSummary = exports.output = void 0;
const output_1 = require("../lint/output");
const output_2 = require("../lint/output");
const screen_1 = require("../../screen");
function output(result) {
    (0, output_1.output)(result.lintResult, {
        skipSummary: true,
    });
    (0, screen_1.print)();
    if (result.rules) {
        for (const rule of result.rules) {
            if (rule.namespace) {
                (0, screen_1.print)(`${screen_1.OBJECT_ICONS.rule.get()} [${rule.kind}] Namespace: ${rule.namespace}, ${rule.rule}`);
            }
            else {
                (0, screen_1.print)(`${screen_1.OBJECT_ICONS.rule.get()} [${rule.kind}] ${rule.rule}`);
            }
            (0, screen_1.print)((0, output_1.produceSourceLine)(rule.source), 3);
            if (!rule.compiled) {
                (0, screen_1.printFailLine)(`Failed to compile`, 3);
            }
            (0, screen_1.printErrors)(rule.errors, 6);
            if (rule.pass && rule.compiled) {
                if (rule.passed.length > 0) {
                    (0, screen_1.printPassLine)('Rule passed', 3);
                }
                else {
                    (0, screen_1.printInactivePassLine)('Rule passed. No manifests found to check.', 3);
                }
            }
            if (!rule.pass) {
                (0, screen_1.printFailLine)('Rule failed', 3);
            }
            if (rule.passed.length > 0) {
                (0, screen_1.printSectionTitle)('Passed:', 3);
                for (const manifest of rule.passed) {
                    (0, output_2.outputManifest)(manifest.manifest, screen_1.STATUS_ICONS.passed, 6);
                    (0, screen_1.print)((0, output_1.produceSourceLine)(manifest.source), 9);
                }
            }
            if (rule.violations) {
                (0, screen_1.printSectionTitle)('Violations:', 3);
                for (const violation of rule.violations) {
                    (0, output_2.outputManifest)(violation.manifest, screen_1.STATUS_ICONS.failed, 6);
                    (0, screen_1.print)((0, output_1.produceSourceLine)(violation.source), 9);
                    (0, screen_1.printErrors)(violation.errors, 9);
                    (0, screen_1.printWarnings)(violation.warnings, 9);
                }
            }
            (0, screen_1.print)();
        }
        (0, screen_1.print)();
    }
    (0, output_1.outputLintSummary)(result.lintResult);
    (0, screen_1.print)();
    outputGuardSummary(result);
    (0, output_1.outputLintResult)(result.lintResult);
    (0, screen_1.printProcessStatus)(result.success, 'Guard');
}
exports.output = output;
function outputGuardSummary(result) {
    (0, screen_1.printSectionTitle)('Guard Summary');
    (0, screen_1.print)(`Rules: ${result.counters.rules.total}`, 4);
    (0, screen_1.printSummaryCounter)(screen_1.STATUS_ICONS.passed, 'Rules Passed', result.counters.rules.passed);
    (0, screen_1.printSummaryCounter)(screen_1.STATUS_ICONS.failed, 'Rules Failed', result.counters.rules.failed);
    (0, screen_1.printSummaryCounter)(screen_1.STATUS_ICONS.error, 'Rules With Errors', result.counters.rules.withErrors);
    (0, screen_1.printSummaryCounter)(screen_1.STATUS_ICONS.warning, 'Rules With Warnings', result.counters.rules.withWarnings);
    (0, screen_1.print)(`Manifests: ${result.counters.manifests.total}`, 4);
    (0, screen_1.printSummaryCounter)(screen_1.OBJECT_ICONS.manifest, 'Manifests Processed', result.counters.manifests.processed);
    (0, screen_1.printSummaryCounter)(screen_1.STATUS_ICONS.passed, 'Manifests Passed', result.counters.manifests.passed);
    (0, screen_1.printSummaryCounter)(screen_1.STATUS_ICONS.failed, 'Manifests with Errors', result.counters.manifests.withErrors);
    (0, screen_1.printSummaryCounter)(screen_1.STATUS_ICONS.warning, 'Manifests With Warnings', result.counters.manifests.withWarnings);
}
exports.outputGuardSummary = outputGuardSummary;
//# sourceMappingURL=output.js.map