"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ManifetsLoader = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
const axios_1 = __importDefault(require("axios"));
const glob_1 = __importDefault(require("glob"));
const fs = __importStar(require("fs"));
const Path = __importStar(require("path"));
const yaml_1 = __importDefault(require("yaml"));
const stream_1 = require("../utils/stream");
const spinner_1 = require("../screen/spinner");
const path_1 = require("../utils/path");
class ManifetsLoader {
    constructor(logger, manifestPackage, options) {
        var _a;
        this._logger = logger.sublogger('ManifetsLoader');
        this._manifestPackage = manifestPackage;
        options = options !== null && options !== void 0 ? options : {};
        this._options = {
            ignoreNonK8s: (_a = options.ignoreNonK8s) !== null && _a !== void 0 ? _a : false
        };
        this._logger.info("setup. options: ", this._options);
    }
    get manifestPackage() {
        return this._manifestPackage;
    }
    load(fileOrPatternOrUrls) {
        this._logger.info("[load] fileOrPatternOrUrl: ", fileOrPatternOrUrls);
        if (fileOrPatternOrUrls.length === 0) {
            return the_promise_1.Promise.resolve();
        }
        const spinner = (0, spinner_1.spinOperation)('Loading manifests...');
        return the_promise_1.Promise.resolve()
            .then(() => {
            return this._loadMany(fileOrPatternOrUrls);
        })
            .then(() => {
            spinner.complete('Manifests loaded.');
        });
    }
    loadFromStream() {
        this._logger.info("[_loadFromStream] ");
        const source = this._manifestPackage.getSource("stream", 'stream');
        return (0, stream_1.readFromInputStream)()
            .then((contents) => {
            var _a;
            let manifests = null;
            if (!manifests) {
                try {
                    manifests = parseYaml(contents);
                }
                catch (reason) {
                    this._manifestPackage.sourceError(source, (_a = reason.message) !== null && _a !== void 0 ? _a : 'Error parsing YAML.');
                }
            }
            if (!manifests) {
                try {
                    manifests = [JSON.parse(contents)];
                }
                catch (reason) {
                    manifests = null;
                }
            }
            if (manifests) {
                for (const manifest of manifests) {
                    this._addManifest(source, manifest);
                }
            }
            else {
                this._manifestPackage.sourceError(source, 'Failed to parse manifests from stream.');
            }
        })
            .catch(reason => {
            this._manifestPackage.sourceError(source, 'Failed to fetch manifest. Reason: ' + reason.message);
        });
    }
    _loadMany(fileOrPatternOrUrls) {
        return the_promise_1.Promise.serial(fileOrPatternOrUrls, x => this.loadSingle(x))
            .then(() => { });
    }
    loadSingle(fileOrPatternOrUrl, parentSource) {
        let finalPath = fileOrPatternOrUrl;
        if (parentSource &&
            (parentSource.source.kind === "file" || parentSource.source.kind === "web")) {
            finalPath = (0, path_1.joinPath)(parentSource.source.path, finalPath);
        }
        if ((0, path_1.isWeb)(finalPath)) {
            return this._loadUrl(finalPath);
        }
        else {
            return this._loadFileOrPattern(finalPath);
        }
    }
    _loadFileOrPattern(fileOrPattern) {
        const pattern = this._makeSearchPattern(fileOrPattern);
        this._logger.info("[_loadFileOrPattern] pattern: %s", pattern);
        if (!pattern) {
            return the_promise_1.Promise.resolve([]);
        }
        return the_promise_1.Promise.construct((resolve, reject) => {
            return (0, glob_1.default)(pattern, {
                nodir: true
            }, (err, matches) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(matches);
                }
            });
        })
            .then(files => {
            this._logger.info("[_loadFileOrPattern] files: ", files);
            return the_promise_1.Promise.serial(files, file => this._loadFile(file));
        })
            .then(results => the_lodash_1.default.flatten(results));
    }
    _loadFile(path) {
        const source = this._manifestPackage.getSource("file", path);
        this._logger.info("[_loadFile] path: %s", path);
        const contents = fs.readFileSync(path, { encoding: 'utf8' });
        return this._parseContents(source, path, contents);
    }
    _loadUrl(url) {
        const source = this._manifestPackage.getSource("web", url);
        this._logger.info("[_loadUrl] url: %s", url);
        return the_promise_1.Promise.resolve()
            .then(() => axios_1.default.get(url))
            .then(({ data }) => {
            const contents = data.toString();
            return this._parseContents(source, url, contents);
        })
            .catch(reason => {
            this._manifestPackage.sourceError(source, 'Failed to fetch manifest. Reason: ' + reason.message);
            return [];
        });
    }
    _parseContents(source, path, contents) {
        var _a, _b;
        this._logger.info("[_parseContents] path: %s", path);
        const manifests = [];
        const extension = Path.extname(path);
        if (extension === '.yaml' || extension === '.yml') {
            let configs = [];
            try {
                configs = parseYaml(contents);
            }
            catch (reason) {
                this._manifestPackage.sourceError(source, (_a = reason.message) !== null && _a !== void 0 ? _a : 'Error parsing YAML.');
            }
            if (configs) {
                this._logger.info("[_parseContents] Manifest Count: %s", configs.length);
                this._logger.silly("[_parseContents] Manifests: ", configs);
                for (const config of configs) {
                    const manifest = this._addManifest(source, config);
                    if (manifest) {
                        manifests.push(manifest);
                    }
                }
            }
            else {
                this._logger.info("[_parseContents] No Manifests Found.");
            }
        }
        else if (extension === '.json') {
            let configs;
            try {
                configs = JSON.parse(contents);
            }
            catch (reason) {
                this._manifestPackage.sourceError(source, (_b = reason.message) !== null && _b !== void 0 ? _b : 'Error parsing JSON');
            }
            if (configs) {
                const manifest = this._addManifest(source, configs);
                if (manifest) {
                    manifests.push(manifest);
                }
            }
        }
        else {
            this._manifestPackage.sourceError(source, 'Unknown extension. Should be one of: .yaml, .yml or .json');
        }
        if (source.success) {
            if (source.contents.length === 0) {
                if (!this._options.ignoreNonK8s) {
                    this._manifestPackage.sourceError(source, 'Contains no manifests');
                }
            }
        }
        return manifests;
    }
    _addManifest(source, config) {
        this._logger.silly("[_addManifest] file: %s, manifest:", source.source.path, config);
        if (!config) {
            return null;
        }
        const k8sObject = config;
        const errors = this._checkK8sManifest(k8sObject);
        if (errors.length > 0) {
            if (!this._options.ignoreNonK8s) {
                this._manifestPackage.sourceErrors(source, errors);
            }
            return null;
        }
        return this._manifestPackage.addManifest(source, k8sObject);
    }
    _checkK8sManifest(k8sObject) {
        const errors = [];
        if (!k8sObject.apiVersion) {
            errors.push('Not a Kubernetes manifest. apiVersion is missing.');
        }
        if (!k8sObject.kind) {
            errors.push('Not a Kubernetes manifest. kind is missing.');
        }
        return errors;
    }
    _makeSearchPattern(fileOrPattern) {
        if (fs.existsSync(fileOrPattern)) {
            const stats = fs.statSync(fileOrPattern);
            if (stats.isDirectory()) {
                return Path.join(fileOrPattern, '**/*[.yaml|.yml]');
            }
            if (stats.isFile()) {
                return fileOrPattern;
            }
        }
        else {
            return fileOrPattern;
        }
        return null;
    }
}
exports.ManifetsLoader = ManifetsLoader;
function parseYaml(contents) {
    // console.log(">>>>>>>>>>>>>>>")
    // console.log(contents);
    // console.log("<<<<<<<<>>>>>>>>*")
    const result = yaml_1.default.parseAllDocuments(contents, {});
    if (!result) {
        return [];
    }
    const jsons = result.map(x => x.toJS({})); // emptySourceAsObject: false 
    // console.log("***************************")
    // console.log(jsons);
    // console.log("***************************")
    return jsons;
}
//# sourceMappingURL=manifests-loader.js.map